@@ -3,6 +3,8 @@
 //! to be executed as part of a flow.
 //!
 use std::fmt::Debug;
+use std::error;
+use std::fmt;
 use std::panic::{RefUnwindSafe, UnwindSafe};
 
 use serde_json::Value;
@@ -15,6 +17,28 @@ pub const RUN_AGAIN: RunAgain = true;
 /// Use `DONT_RUN_AGAIN` to indicate that a function should not be executed more times
 pub const DONT_RUN_AGAIN: RunAgain = false;
 
+/// A `RuntimeError` is an error that occurred during the running of a Function's `Implementation`
+#[derive(Debug, Clone)]
+pub enum RuntimeError {
+    /// There was an error with the inputs to the `Implementation`
+    InputError(String),
+    /// There was an error during the execution of the `Implementation`
+    Execution(String)
+}
+
+impl fmt::Display for RuntimeError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{:?}", self)
+    }
+}
+
+impl error::Error for RuntimeError {
+    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
+        // Generic error, underlying cause isn't tracked.
+        None
+    }
+}
+
 /// An implementation runs with an array of inputs and returns a value (or null) and a
 /// bool indicating if it should be ran again.
 ///
@@ -25,7 +49,7 @@ pub const DONT_RUN_AGAIN: RunAgain = false;
 /// Here is an example implementation of this trait:
 ///
 /// ```
-/// use flow_impl::{Implementation, RUN_AGAIN, RunAgain};
+/// use flow_impl::{Implementation, RUN_AGAIN, RunAgain, RuntimeError};
 /// use serde_json::Value;
 /// use serde_json::json;
 ///
@@ -36,7 +60,7 @@ pub const DONT_RUN_AGAIN: RunAgain = false;
 ///     A compare operator that takes two numbers and outputs the comparisons between them
 /// */
 /// impl Implementation for Compare {
-///     fn run(&self, mut inputs: &[Value]) -> (Option<Value>, RunAgain) {
+///     fn run(&self, mut inputs: &[Value]) -> (Result<Option<Value>, RuntimeError>, RunAgain) {
 ///         let left = inputs[0].as_i64().unwrap();
 ///         let right = inputs[1].as_i64().unwrap();
 ///
@@ -48,11 +72,11 @@ pub const DONT_RUN_AGAIN: RunAgain = false;
 ///                     "gte" : left >= right
 ///                 });
 ///
-///         (None, RUN_AGAIN)
+///         (Ok(None), RUN_AGAIN)
 ///     }
 /// }
 /// ```
 pub trait Implementation : RefUnwindSafe + UnwindSafe + Sync + Send + Debug {
     /// The `run` method is used to execute the implementation
-    fn run(&self, inputs: &[Value]) -> (Option<Value>, RunAgain);
+    fn run(&self, inputs: &[Value]) -> (Result<Option<Value>, RuntimeError>, RunAgain);
 }
\ No newline at end of file
